"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@babel/core");
const util_1 = require("util");
const getConfigExports_1 = require("./utils/getConfigExports");
const prepareConfig_1 = require("./utils/prepareConfig");
const configsCache = new Map();
const exportsCache = new Map();
function getConfigs(modules) {
    if (configsCache.size === 0) {
        for (const config of prepareConfig_1.prepareConfig(modules)) {
            configsCache.set(config.name, config);
        }
    }
    return configsCache;
}
function getExports(name, modules) {
    let pkgExports = exportsCache.get(name);
    if (!pkgExports) {
        const config = getConfigs(modules).get(name);
        if (!config) {
            pkgExports = new Map();
        }
        else {
            pkgExports = getConfigExports_1.getConfigExports(config);
        }
        exportsCache.set(name, pkgExports);
    }
    return pkgExports;
}
function plugin() {
    return {
        visitor: {
            ImportDeclaration(declaration, { opts }) {
                const { source, specifiers, importKind } = declaration.node;
                if (!opts ||
                    !opts.modules ||
                    !Array.isArray(opts.modules) ||
                    opts.modules.length === 0) {
                    return;
                }
                if (importKind === 'type') {
                    return;
                }
                if (specifiers.length === 0) {
                    return;
                }
                const exports = getExports(source.value, opts.modules);
                if (exports.size === 0) {
                    return;
                }
                for (const specifier of specifiers) {
                    if (core_1.types.isImportNamespaceSpecifier(specifier)) {
                        console.warn(util_1.format('babel-plugin-direct-import: Can not optimize `import * as %s from "%s"`.'.concat('\n', 'See plugin limitations https://git.io/vFDOO for more details.'), specifier.local.name, source.value));
                        continue;
                    }
                    const moduleName = core_1.types.isImportDefaultSpecifier(specifier)
                        ? 'default'
                        : core_1.types.isIdentifier(specifier.imported)
                            ? specifier.imported.name
                            : specifier.imported.value;
                    const moduleExports = exports.get(moduleName);
                    if (moduleExports) {
                        declaration.node.specifiers = declaration.node.specifiers.filter((x) => x !== specifier);
                        declaration.insertBefore(core_1.types.importDeclaration([
                            moduleExports.internal === '*'
                                ? core_1.types.importNamespaceSpecifier(core_1.types.identifier(specifier.local.name))
                                : moduleExports.internal === 'default'
                                    ? core_1.types.importDefaultSpecifier(core_1.types.identifier(specifier.local.name))
                                    : core_1.types.importSpecifier(core_1.types.identifier(specifier.local.name), core_1.types.identifier(moduleExports.external)),
                        ], core_1.types.stringLiteral(moduleExports.source)));
                    }
                }
                if (declaration.node.specifiers.length === 0) {
                    declaration.remove();
                }
            },
        },
    };
}
exports.default = plugin;
