"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.prepareConfig = void 0;
const asserts_1 = require("./asserts");
function prepareConfig(args) {
    const configs = Array.isArray(args) ? args : [args];
    return configs.map((options) => {
        if (typeof options === 'string') {
            return { name: options };
        }
        const { name, indexFile, ...unknownOptions } = options;
        asserts_1.assert(typeof name === 'string', '{ name } expected to be a string');
        asserts_1.assert(!!name, '{ name } is empty');
        if (indexFile != null) {
            asserts_1.assert(typeof indexFile === 'string', '{ indexFile } expected to be a string');
            asserts_1.assert(!!indexFile, '{ indexFile } is empty');
        }
        if (indexFile != null && indexFile !== name) {
            const [nameID] = name.split('/');
            const [indexFileID] = indexFile.split('/');
            asserts_1.assert(nameID === indexFileID, "index file '%s' must belong to '%s' package", indexFile, name);
        }
        const unknownOptionsKeys = Object.keys(unknownOptions);
        asserts_1.assert(unknownOptionsKeys.length === 0, 'contains unknown keys { %s }', unknownOptionsKeys.join(', '));
        return options;
    });
}
exports.prepareConfig = prepareConfig;
